using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Timers;
using Tao.OpenGl;
using Tao.Platform.Windows;

namespace SimpleSolarSystem
{
    public partial class SimpleSolarSystemForm : Form
    {
        #region Attributes
        public Icon newIcon;
        private double diffEartheta = 0.3;
       
       
        private int viewportW = 800;
        private int viewportH = 600;
        private Rectangle rec;
        private bool quit = false;
        private double winX = 3.0;
        private double winY = 3.0;

        //angles for planets
        public static double thetaEarthSun = 0.0;
        public static double thetaMoonEarth = 0.0;
        public static double thetaMercurySun = 0.0;
        public static double thetaEarthAxis = 0.0;
        public static double thetaSaturnSun = 0.0;
        public static int rounds = 2;

        //Light values
        private float []lightPos0={0.0f,0.2f,0.0f,1.0f};
	    private float []model_ambient={0.3f,0.3f,0.3f,1.0f};	   
    	
	    //GLfloat mat_amb[]={0.4,0.4,0.4,1.0};
        private float[] mat_diff ={ 0.9f, 0.0f, 0.0f, 1.0f };
        private float[] mat_spec ={ 0.6f, 0.6f, 0.6f, 1.0f };
        private float[] emission ={ 0.8f, 0.8f, 0.8f, 1.0f };

	    private float no_shininess=0.0f;
	    private float low_shininess=20.0f;
	    private float high_shininess=100.0f;
        #endregion

        public SimpleSolarSystemForm()
        {
            InitializeComponent();
            glControl.InitializeContexts();
            this.SetBounds(0, 0, viewportW, viewportH);
            initializeOpenGl();
            getIconFromBitmap();
        }
        public void getIconFromBitmap()
        {
            Bitmap myBitmap = new Bitmap(@"..\..\Images\rose12.jpg");
            IntPtr Hicon = myBitmap.GetHicon();
            newIcon = Icon.FromHandle(Hicon);
            this.Icon = newIcon;
            //DestroyIcon(newIcon.Handle);
        }
        public void initializeOpenGl()
        {
           
            Gl.glViewport(0, 0, viewportW, viewportH);
            Gl.glMatrixMode(Gl.GL_PROJECTION);
            Gl.glLoadIdentity();

            //Gl.glOrtho(-winX * (8.0 / 6.0), winX * (8.0 / 6.0), -winY, winY, -20, 20);
            Glu.gluPerspective(90, 1.4, 1, 15);
            Gl.glMatrixMode(Gl.GL_MODELVIEW);
            Gl.glLoadIdentity();
            Glu.gluLookAt(0.0, 1.3, 4.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);

            Gl.glClearColor(0, 0, 0, 1);
            Gl.glClear(Gl.GL_COLOR_BUFFER_BIT | Gl.GL_DEPTH_BUFFER_BIT);
            Gl.glEnable(Gl.GL_DEPTH_TEST);
            Gl.glShadeModel(Gl.GL_SMOOTH);
            initLights();
        }
        public void initLights()
        {
            Gl.glLightfv(Gl.GL_LIGHT0,Gl.GL_POSITION,lightPos0);

            Gl.glLightModelfv(Gl.GL_LIGHT_MODEL_AMBIENT, model_ambient);
            Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_DIFFUSE, mat_diff);
            Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_SPECULAR, mat_spec);
            Gl.glMaterialf(Gl.GL_FRONT, Gl.GL_SHININESS, high_shininess);
            Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_EMISSION, emission);
            Gl.glColorMaterial(Gl.GL_FRONT_AND_BACK, Gl.GL_DIFFUSE);
            Gl.glEnable(Gl.GL_COLOR_MATERIAL);   

            Gl.glEnable(Gl.GL_LIGHTING);
            Gl.glEnable(Gl.GL_LIGHT0);

            Gl.glShadeModel(Gl.GL_SMOOTH);          
        }   

        private void glControl_Paint(object sender, PaintEventArgs e)
        {
            Gl.glClear(Gl.GL_COLOR_BUFFER_BIT | Gl.GL_DEPTH_BUFFER_BIT);
            renderSolarSystem();
            Thread.Sleep(10);
            if (quit == true)
            {
                Application.Exit();
            }
            else
                glControl.Invalidate();
           
        }

        private void glControl_Resize(object sender, EventArgs e)
        {
            //rec = ClientRectangle;
            //viewportW = rec.Width;
            //viewportH = rec.Height;

            //Gl.glViewport(0, 0, viewportW, viewportH);
            //Gl.glMatrixMode(Gl.GL_PROJECTION);
            //Gl.glLoadIdentity();

            //if (viewportW > viewportH)
            //    Gl.glOrtho(-winX * viewportW / viewportH, winX * viewportW / viewportH, -winY, winY, -10, 10);
            //else
            //    Gl.glOrtho(-winX, winX, winY * viewportH / viewportW, -winY * viewportH / viewportW, -10, 10);
           
            //Gl.glMatrixMode(Gl.GL_MODELVIEW);
            //Gl.glLoadIdentity();
            //Glu.gluLookAt(0.0, 1.3, 4.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
           
            //glControl.Refresh();
        }

        public void renderSolarSystem()
        {
            Gl.glClear(Gl.GL_COLOR_BUFFER_BIT | Gl.GL_DEPTH_BUFFER_BIT);
            float []emission={1.0f,0.0f,0.0f,1.0f};
            Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_EMISSION, emission);
            Gl.glColor4f(1.0f,0.0f,0.0f,1.0f);
            Sphere(0.4, 50, 50);
          
            emission[0] = 0.3f;
            emission[1] = 0.3f;
            emission[2] = 0.3f;
            Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_EMISSION, emission);

            Gl.glPushMatrix();
                Gl.glRotated(thetaMercurySun, 0.0, 1.0, 0.0);
                Gl.glTranslated(0.4, 0.0, 0.4);               
                Gl.glColor4f(1.0f,0.0f,1.0f,1.0f);		//Gl.glMaterialfv(GL_FRONT,Gl.gl_DIFFUSE,mat_diff);
                Sphere(0.08, 20, 20);			// Planet I Mercury
            Gl.glPopMatrix();

            Gl.glPushMatrix();
                Gl.glRotated(thetaEarthSun, 0.0, 1.0, 0.0);
                Gl.glTranslated(1.0, 0.0, 1.0);
                Gl.glRotated(thetaEarthAxis, 0.0, 1.0, 0.0);
                Gl.glColor4f(0.0f,0.8f,0.0f,1.0f);
                Sphere(0.25, 15, 15);//50,50);//Planet II Earth	
                Gl.glColor4f(0.0f, 0.0f, 0.0f, 1.0f);
                WireSphere(0.251, 15, 15);
            Gl.glPopMatrix();

            Gl.glPushMatrix();               
                Gl.glColor4f(0.8f,0.8f,0.8f,1.0f);	//Gl.glMaterialfv(Gl.gl_FRONT,Gl.gl_DIFFUSE,mat_diff);
                Gl.glRotated(thetaEarthSun, 0.0, 1.0, 0.0);		//Rotation around sun
                Gl.glTranslated(1.0, 0.0, 1.0);						//Translation from sun to earth
                Gl.glRotated(thetaMoonEarth, 0.0, 1.0, 0.0);
                Gl.glTranslated(0.25, 0.0, 0.25);					//Added translation for moon
                Sphere(0.06, 20, 20);					//Moon of Earth				
            Gl.glPopMatrix();

            Gl.glPushMatrix();
                Gl.glRotated(thetaSaturnSun, 0.0, 1.0, 0.0);		//Rotation around sun
                Gl.glTranslated(2.5, 0.0, 0.0);
                saturn();									//Saturn with Rings
            Gl.glPopMatrix();

            thetaEarthSun = thetaEarthSun + diffEartheta;
            thetaMoonEarth = thetaMoonEarth + 1.0;
            thetaMercurySun = thetaMercurySun + 1;
            thetaEarthAxis = thetaEarthAxis + 12;
            thetaSaturnSun = thetaSaturnSun + 0.8;

        }
        public void saturn()
        {
	        double r=0.25;	
	       
	        Gl.glColor4f(1.0f,0.3f,0.0f,1.0f);
	        Sphere(r,50,50);				//Saturn
            Gl.glColor4f(1.0f, 0.0f, 0.0f, 1.0f);
            Gl.glPushMatrix();
                Gl.glRotated(70, 1, 0, 0);              
		        Disk(0.3, 0.55, 40,40);
            Gl.glPopMatrix();
        	
        }
        #region Objects rendered(Sphere,Disk,Cylinder,Cone,Cube)
            public void Sphere(double radius, int slices, int stacks)
            {
                Glu.GLUquadric quad = Glu.gluNewQuadric();
                Glu.gluQuadricDrawStyle(quad, Glu.GLU_FILL);//GLU_POINT,GLU_LINE,GLU_SILHOUETTE
                Glu.gluQuadricNormals(quad, Glu.GLU_SMOOTH);
                //Glu.gluQuadricOrientation(quad, Glu.GLU_INSIDE);
                Glu.gluSphere(quad, radius, slices, stacks);
            }
            public void WireSphere(double radius, int slices, int stacks)
            {
                Glu.GLUquadric quad = Glu.gluNewQuadric();
                Glu.gluQuadricDrawStyle(quad, Glu.GLU_LINE);//GLU_POINT,GLU_LINE,GLU_SILHOUETTE
                Glu.gluQuadricNormals(quad, Glu.GLU_SMOOTH);
                //Glu.gluQuadricOrientation(quad, Glu.GLU_INSIDE);
                Gl.glPushMatrix();
                    Gl.glRotated(90, 1, 0, 0);
                    Glu.gluSphere(quad, radius, slices, stacks);
                Gl.glPopMatrix();
            }
            public void Disk(double inRadius, double outRadius, int slices, int stacks)
            {
                Glu.GLUquadric quad = Glu.gluNewQuadric();
                
                Glu.gluQuadricDrawStyle(quad, Glu.GLU_FILL);//GLU_POINT,GLU_LINE,GLU_SILHOUETTE
                Glu.gluQuadricNormals(quad, Glu.GLU_SMOOTH);
                Glu.gluQuadricOrientation(quad, Glu.GLU_INSIDE);
                Glu.gluDisk(quad, inRadius, outRadius, slices, stacks);
            }
            public void WireDisk(double inRadius, double outRadius, int slices, int stacks)
            {
                Glu.GLUquadric quad = Glu.gluNewQuadric();
                Glu.gluQuadricDrawStyle(quad, Glu.GLU_LINE);//GLU_POINT,GLU_LINE,GLU_SILHOUETTE
                Glu.gluQuadricNormals(quad, Glu.GLU_SMOOTH);
                Glu.gluDisk(quad, inRadius, outRadius, slices, stacks);
            }
            public void Cylinder(double baseRadius, double topRadius, double height, int slices, int stacks)
            {
                Glu.GLUquadric quad = Glu.gluNewQuadric();
                Glu.gluQuadricDrawStyle(quad, Glu.GLU_FILL);//GLU_POINT,GLU_LINE,GLU_SILHOUETTE
                Glu.gluQuadricNormals(quad, Glu.GLU_FLAT);//SMOOTH);
                Glu.gluCylinder(quad, baseRadius, topRadius, height, slices, stacks);
            }
            public void WireCylinder(double baseRadius, double topRadius, double height, int slices, int stacks)
            {
                Glu.GLUquadric quad = Glu.gluNewQuadric();
                Glu.gluQuadricDrawStyle(quad, Glu.GLU_LINE);//GLU_POINT,GLU_LINE,GLU_SILHOUETTE
                Glu.gluQuadricNormals(quad, Glu.GLU_FLAT);//SMOOTH);
                Glu.gluCylinder(quad, baseRadius, topRadius, height, slices, stacks);
            }
            public void Cone(double baseRadius, double height, int slices, int stacks)
            {
                Glu.GLUquadric quad = Glu.gluNewQuadric();
                Glu.gluQuadricDrawStyle(quad, Glu.GLU_FILL);//GLU_POINT,GLU_LINE,GLU_SILHOUETTE
                Glu.gluQuadricNormals(quad, Glu.GLU_SMOOTH);
                Glu.gluCylinder(quad, baseRadius, 0.0, height, slices, stacks);
            }
            public void WireCone(double baseRadius, double height, int slices, int stacks)
            {
                Glu.GLUquadric quad = Glu.gluNewQuadric();
                Glu.gluQuadricDrawStyle(quad, Glu.GLU_LINE);//GLU_POINT,GLU_LINE,GLU_SILHOUETTE
                Glu.gluQuadricNormals(quad, Glu.GLU_SMOOTH);
                Glu.gluCylinder(quad, baseRadius, 0.0, height, slices, stacks);
            }
            public void Cube(double side)
        {
            double hs = side / 2.0;
            double[,] vertex = {{hs,hs,hs},
                                {hs,hs,-hs},
                                {-hs,hs,-hs},
                                {-hs,hs,hs},
                                {hs,-hs,hs},
                                {hs,-hs,-hs},
                                {-hs,-hs,-hs},
                                {-hs,-hs,hs}};
            int[,] face ={  {0,1,2,3},
                            {7,6,5,4},           
                            {0,4,5,1},
                            {6,7,3,2},
                            {0,3,7,4},
                            {2,1,5,6}};
            double[,] normal ={ {0,1,0},
                                {0,-1,0},
                                {1,0,0},
                                {-1,0,0},
                                {0,0,1},
                                {0,0,-1}};
            Gl.glBegin(Gl.GL_QUADS);
            for (int f = 0; f < 6; f++)
            {
                for (int v = 0; v < 4; v++)
                {
                    Gl.glNormal3d(normal[f, 0], normal[f, 1], normal[f, 2]);
                    Gl.glVertex3d(vertex[face[f, v], 0], vertex[face[f, v], 1], vertex[face[f, v], 2]);
                }
            }
            Gl.glEnd();
        }
            public void WireCube(double side)
            {
                double hs = side / 2.0;
                double[,] vertex = {{hs,hs,hs},
                                    {hs,hs,-hs},
                                    {-hs,hs,-hs},
                                    {-hs,hs,hs},
                                    {hs,-hs,hs},
                                    {hs,-hs,-hs},
                                    {-hs,-hs,-hs},
                                    {-hs,-hs,hs}};
                int [,] edge ={{0,1},{1,2},{2,3},{3,0},
                                {7,6},{6,5},{5,4},{4,7},
                                {7,3},{4,0},{5,1},{6,2}};
                               
                Gl.glBegin(Gl.GL_LINES);
                    for (int e = 0; e < 12; e++)
                    {                      
                        Gl.glVertex3d(vertex[edge[e, 0],0], vertex[edge[e, 0], 1], vertex[edge[e, 0], 2]);
                        Gl.glVertex3d(vertex[edge[e, 1],0], vertex[edge[e, 1], 1], vertex[edge[e, 1], 2]);
                    }
                Gl.glEnd();
            }
        #endregion

        private void glControl_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Escape)
                quit = true;
        }
    }
}